import grabHostNames from "../../utils/grab-host-names";
import apiUpdateUser from "../../functions/api/users/api-update-user";
import { UpdateUserFunctionReturn } from "../../types";

type Param = {
    key?: string;
    database: string;
    updatedUserId: string | number;
    payload: { [s: string]: any };
    user_id?: boolean;
    useLocal?: boolean;
};

/**
 * # Update User
 */
export default async function updateUser({
    key,
    payload,
    database,
    user_id,
    updatedUserId,
    useLocal,
}: Param): Promise<UpdateUserFunctionReturn> {
    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;

    if (useLocal) {
        return await apiUpdateUser({
            payload: payload,
            dbFullName: database,
            updatedUserId,
        });
    }

    /**
     * Make https request
     *
     * @description make a request to datasquirel.com
     */
    const httpResponse = await new Promise((resolve, reject) => {
        const reqPayload = JSON.stringify({
            payload,
            database,
            updatedUserId,
        });

        const httpsRequest = scheme.request(
            {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    "Content-Length": Buffer.from(reqPayload).length,
                    Authorization:
                        process.env.DSQL_FULL_ACCESS_API_KEY ||
                        process.env.DSQL_API_KEY ||
                        key,
                },
                port,
                hostname: host,
                path: `/api/user/${
                    user_id || grabedHostNames.user_id
                }/update-user`,
            },

            /**
             * Callback Function
             *
             * @description https request callback
             */
            (response) => {
                var str = "";

                response.on("data", function (chunk) {
                    str += chunk;
                });

                response.on("end", function () {
                    resolve(JSON.parse(str));
                });

                response.on("error", (err) => {
                    reject(err);
                });
            }
        );
        httpsRequest.write(reqPayload);
        httpsRequest.end();
    });

    return httpResponse as UpdateUserFunctionReturn;
}
