import serverError from "./serverError";
import addDbEntry from "./db/addDbEntry";
import createDbFromSchema from "../../shell/createDbFromSchema";
import grabNewUsersTableSchema from "./grabNewUsersTableSchema";
import {
    grabPrimaryRequiredDbSchema,
    writeUpdatedDbSchema,
} from "../../shell/createDbFromSchema/grab-required-database-schemas";
import dbHandler from "./dbHandler";

type Param = {
    userId?: number | string;
    database: string;
    payload?: { [s: string]: any };
    dbId: string | number;
};

/**
 * # Add User Table to Database
 */
export default async function addUsersTableToDb({
    userId,
    database,
    payload,
    dbId,
}: Param): Promise<any> {
    try {
        const dbFullName = database;

        const userPreset = grabNewUsersTableSchema({ payload });
        if (!userPreset) throw new Error("Couldn't Get User Preset!");

        let targetDatabase = grabPrimaryRequiredDbSchema({
            dbId,
            userId,
        });

        if (!targetDatabase) {
            throw new Error("Couldn't Find Target Database!");
        }

        let existingTableIndex = targetDatabase?.tables.findIndex(
            (table: any) => table.tableName === "users"
        );

        if (typeof existingTableIndex == "number" && existingTableIndex > 0) {
            targetDatabase.tables[existingTableIndex] = userPreset;
        } else {
            targetDatabase.tables.push(userPreset);
        }

        writeUpdatedDbSchema({ dbSchema: targetDatabase, userId });

        const targetDb = (await dbHandler({
            query: `SELECT id FROM user_databases WHERE user_id=? AND db_slug=?`,
            values: [userId, database],
        })) as any[];

        if (targetDb?.[0]) {
            const newTableEntry = await addDbEntry({
                dbFullName: "datasquirel",
                tableName: "user_database_tables",
                data: {
                    user_id: userId,
                    db_id: targetDb[0].id,
                    db_slug: targetDatabase.dbSlug,
                    table_name: "Users",
                    table_slug: "users",
                },
            });
        }

        const dbShellUpdate = await createDbFromSchema({
            userId,
            targetDatabase: dbFullName,
        });

        return `Done!`;
    } catch (/** @type {any} */ error: any) {
        console.log(`addUsersTableToDb.ts ERROR: ${error.message}`);

        serverError({
            component: "addUsersTableToDb",
            message: error.message,
            user: { id: userId },
        });
        return error.message;
    }
}
