import fs from "fs";
import grabDirNames from "./names/grab-dir-names";
import type { ConnectionConfig } from "mariadb";

type Return = ConnectionConfig["ssl"] | undefined;

/**
 * # Grab SSL
 */
export default function grabDbSSL(): Return {
    const { maxscaleSSLCaCertFile } = grabDirNames();

    const caFilePath = process.env.DSQL_SSL_CA_CERT || maxscaleSSLCaCertFile;

    if (!caFilePath?.match(/./)) {
        return undefined;
    }

    if (!fs.existsSync(caFilePath)) {
        console.log(`${caFilePath} does not exist`);
        return undefined;
    }

    return {
        ca: fs.readFileSync(caFilePath),
        rejectUnauthorized: false,
    };
}
